// @dart=2.9

import 'dart:convert';
import 'dart:typed_data';

import 'package:feedback/feedback.dart';

import 'package:http/http.dart' as http;
import 'package:http_parser/http_parser.dart';

/// This is just an example and not intended as a production ready solution
///
/// See https://docs.gitlab.com/ce/api/projects.html#upload-a-file
/// and https://docs.gitlab.com/ee/api/issues.html#new-issue
OnFeedbackCallback createGitLabIssue(GitLabConfig config) {
  return (
    String feedback,
    Uint8List pngImage,
  ) =>
      _gitlabFeedback(feedback, pngImage, config);
}

Future<void> _gitlabFeedback(
  String feedback,
  Uint8List pngImage,
  GitLabConfig config,
) async {
  final file = await uploadFile(config, pngImage);

  final issue = IssuePostContent(
    'feedback_${DateTime.now().toIso8601String()}',
    '$feedback  ${file.markdown}',
    'feedback',
  );

  final response = await http.post(
      Uri.parse(config.issueUrl.toString() + issue.toUrlParameter()),
      headers: {
        'PRIVATE-TOKEN': config.token,
      });

  print(response.statusCode);
}

Future<GitLabFileUploadResponse> uploadFile(
  GitLabConfig config,
  Uint8List pngImage,
) async {
  final uri = config.fileUploadUrl;
  final request = http.MultipartRequest('POST', uri)
    ..headers['PRIVATE-TOKEN'] = config.token
    ..fields['id'] = config.projectId.toString()
    ..files.add(http.MultipartFile.fromBytes(
      'file',
      pngImage,
      filename: 'screenshot.png',
      contentType: MediaType('image', 'png'),
    ));
  final response = await request.send();
  final responseString = await response.stream.bytesToString();
  return GitLabFileUploadResponse.fromJson(
    json.decode(responseString) as Map<String, dynamic>,
  );
}

class GitLabConfig {
  GitLabConfig(this.projectId, this.token);

  final int projectId;
  final String token;

  Uri get issueUrl =>
      Uri.parse('https://gitlab.com/api/v4/projects/$projectId/issues');

  Uri get fileUploadUrl =>
      Uri.parse('https://gitlab.com/api/v4/projects/$projectId/uploads');
}

class IssuePostContent {
  IssuePostContent(this.title, this.description, this.label);

  final String title;
  final String description;
  final String label;

  String toUrlParameter() {
    final map = {
      if (title != null) 'title': title,
      if (description != null) 'description': description,
      if (label != null) 'label': label,
    };
    // ignore: prefer_interpolation_to_compose_strings
    return '?' +
        map.entries
            .map((entry) =>
                '${entry.key}=${Uri.encodeQueryComponent(entry.value)}')
            .join('&');
  }
}

class GitLabFileUploadResponse {
  GitLabFileUploadResponse({this.alt, this.url, this.fullPath, this.markdown});

  GitLabFileUploadResponse.fromJson(Map<String, dynamic> json) {
    alt = json['alt'] as String;
    url = json['url'] as String;
    fullPath = json['full_path'] as String;
    markdown = json['markdown'] as String;
  }
  String alt;
  String url;
  String fullPath;
  String markdown;
}
